@@ -167,7 +167,8 @@ class OpCreateInstance(OpCode):
     "wait_for_sync", "ip_check", "mac",
     "kernel_path", "initrd_path", "hvm_boot_order", "hvm_acpi",
     "hvm_pae", "hvm_cdrom_image_path", "vnc_bind_address",
-    "iallocator",
+    "iallocator", "hvm_nic_type", "hvm_disk_type",
+    "auto_balance",
     ]
 
 
@@ -273,7 +274,7 @@ class OpQueryInstances(OpCode):
 class OpQueryInstanceData(OpCode):
   """Compute the run-time status of instances."""
   OP_ID = "OP_INSTANCE_QUERY_DATA"
-  __slots__ = ["instances"]
+  __slots__ = ["instances", "static"]
 
 
 class OpSetInstanceParms(OpCode):
@@ -282,7 +283,8 @@ class OpSetInstanceParms(OpCode):
   __slots__ = [
     "instance_name", "mem", "vcpus", "ip", "bridge", "mac",
     "kernel_path", "initrd_path", "hvm_boot_order", "hvm_acpi",
-    "hvm_pae", "hvm_cdrom_image_path", "vnc_bind_address"
+    "hvm_pae", "hvm_cdrom_image_path", "vnc_bind_address",
+    "hvm_nic_type", "hvm_disk_type", "force", "auto_balance",
     ]
 
 
