@@ -25,40 +25,13 @@
 
 # pylint: disable-msg=C0103
 
+import sys
 import os
 
+install_twisted_signal_handlers = True
+
 from twisted.internet.pollreactor import PollReactor
-
-class ReReactor(PollReactor):
-  """A re-startable Reactor implementation.
-
-  """
-  def run(self, installSignalHandlers=1):
-    """Custom run method.
-
-    This is customized run that, before calling Reactor.run, will
-    reinstall the shutdown events and re-create the threadpool in case
-    these are not present (as will happen on the second run of the
-    reactor).
-
-    """
-    if not 'shutdown' in self._eventTriggers:
-      # the shutdown queue has been killed, we are most probably
-      # at the second run, thus recreate the queue
-      self.addSystemEventTrigger('during', 'shutdown', self.crash)
-      self.addSystemEventTrigger('during', 'shutdown', self.disconnectAll)
-    if self.threadpool is not None and self.threadpool.joined == 1:
-      # in case the threadpool has been stopped, re-start it
-      # and add a trigger to stop it at reactor shutdown
-      self.threadpool.start()
-      self.addSystemEventTrigger('during', 'shutdown', self.threadpool.stop)
-
-    return PollReactor.run(self, installSignalHandlers)
-
-
 import twisted.internet.main
-twisted.internet.main.installReactor(ReReactor())
-
 from twisted.spread import pb
 from twisted.internet import reactor
 from twisted.cred import credentials
@@ -97,6 +70,9 @@ class NodeController:
     making the actual call.
 
     """
+    set_keepalive = getattr(obj.broker.transport, 'setTcpKeepAlive', None)
+    if callable(set_keepalive):
+      set_keepalive(True)
     deferred = obj.callRemote(self.parent.procedure, self.parent.args)
     deferred.addCallbacks(self.cb_done, self.cb_err2)
 
@@ -208,6 +184,13 @@ class Client:
     self.results = {}
     self.procedure = procedure
     self.args = args
+    global reactor
+    try:
+      del sys.modules['twisted.internet.reactor']
+    except KeyError:
+      pass
+    reactor = PollReactor()
+    twisted.internet.main.installReactor(reactor)
 
   #--- generic connector -------------
 
@@ -244,7 +227,7 @@ class Client:
 
     """
     if self.nc:
-      reactor.run()
+      reactor.run(install_twisted_signal_handlers)
 
 
 def call_volume_list(node_list, vg_name):
@@ -311,6 +294,18 @@ def call_instance_shutdown(node, instance):
   return c.getresult().get(node, False)
 
 
+def call_instance_migrate(node, instance, target, live):
+  """Migrate an instance.
+
+  This is a single-node call.
+
+  """
+  c = Client("instance_migrate", [instance.name, target, live])
+  c.connect(node)
+  c.run()
+  return c.getresult().get(node, False)
+
+
 def call_instance_reboot(node, instance, reboot_type, extra_args):
   """Reboots an instance.
 
@@ -361,6 +356,18 @@ def call_instance_info(node, instance):
   return c.getresult().get(node, False)
 
 
+def call_instance_migratable(node, instance):
+  """Checks whether the given instance can be migrated.
+
+  This is a single-node call.
+
+  """
+  c = Client("instance_migratable", [instance.ToDict()])
+  c.connect(node)
+  c.run()
+  return c.getresult().get(node, False)
+
+
 def call_all_instances_info(node_list):
   """Returns information about all instances on a given node.
 
@@ -603,6 +610,34 @@ def call_blockdev_find(node, disk):
   return c.getresult().get(node, False)
 
 
+def call_blockdev_close(node, instance_name, disks):
+  """Closes the given block devices.
+
+  This is a single-node call.
+
+  """
+  params = [instance_name, [cf.ToDict() for cf in disks]]
+  c = Client("blockdev_close", params)
+  c.connect(node)
+  c.run()
+  return c.getresult().get(node, False)
+
+
+def call_drbd_reconfig_net(node_list, instance_name, disks,
+                           nodes_ip, multimaster, step):
+  """Re-configures the network connection of drbd disks.
+
+  This is a multi-node call.
+
+  """
+  params = [instance_name, [cf.ToDict() for cf in disks],
+            nodes_ip, multimaster, step]
+  c = Client("drbd_reconfig_net", params)
+  c.connect_list(node_list)
+  c.run()
+  return c.getresult()
+
+
 def call_upload_file(node_list, file_name):
   """Upload a file.
 
@@ -680,6 +715,36 @@ def call_hooks_runner(node_list, hpath, phase, env):
   return result
 
 
+def call_iallocator_runner(node, name, idata):
+  """Call an iallocator on a remote node
+
+  Args:
+    - name: the iallocator name
+    - input: the json-encoded input string
+
+  This is a single-node call.
+
+  """
+  params = [name, idata]
+  c = Client("iallocator_runner", params)
+  c.connect(node)
+  c.run()
+  result = c.getresult().get(node, False)
+  return result
+
+
+def call_blockdev_grow(node, cf_bdev, amount):
+  """Request a snapshot of the given block device.
+
+  This is a single-node call.
+
+  """
+  c = Client("blockdev_grow", [cf_bdev.ToDict(), amount])
+  c.connect(node)
+  c.run()
+  return c.getresult().get(node, False)
+
+
 def call_blockdev_snapshot(node, cf_bdev):
   """Request a snapshot of the given block device.
 
@@ -801,3 +866,15 @@ def call_node_volumes(node_list):
   c.connect_list(node_list)
   c.run()
   return c.getresult()
+
+
+def call_test_delay(node_list, duration):
+  """Sleep for a fixed time on given node(s).
+
+  This is a multi-node call.
+
+  """
+  c = Client("test_delay", [duration])
+  c.connect_list(node_list)
+  c.run()
+  return c.getresult()
